// Stats.java

/**
 * @author Rishi Kacker and Matt Pauker
 * @version 1.0 05/13/2001
 */ 

import java.util.*;
import java.lang.*;
import java.io.*;
import org.w3c.dom.*;

public class Stats {
	
	private XMLHandler data;
	private ArrayList cData;
	
	public Stats() {
		data = new XMLHandler(XMLHandler.NEW,"statistics");
		cData = new ArrayList();
	}
	
	public Stats(String filename) {
		data = new XMLHandler(XMLHandler.LOAD,filename);
		cData = new ArrayList();
	}
	
	public void SaveStats(String filename) {
		data.SaveXML(filename);
	}
	
	//XML tags
	private final static String TIME = "time";
	private final static String CLIENTS = "clients";
	
	public void AppendTimePeriod(Hashtable taskdata) {
		Element c_node = data.CreateElement(TIME);
		
		if (taskdata != null) {
			Enumeration e = taskdata.keys();
			while (e.hasMoreElements()) {
				String key = (String)e.nextElement();
				String value = taskdata.get(key).toString();
				c_node.setAttribute(key,value);
			}
		}
		
		for (int i = 0; i < cData.size(); i++) {
			c_node.appendChild((Element)cData.get(i));
		}
		cData = new ArrayList();
			
		data.Append(c_node);
	}
	
	public void AppendClientData(Client c) {
		cData.add(c.WriteStats(data.GetDocument()));
	}
	
	public void OutputActivity() {
		NodeList nl = data.GetNodeList(TIME);
		if (nl == null) {
			System.out.println("no time entires");
			return;
		}
		
		for (int i = 0; i < nl.getLength(); i++) {
			Node n = nl.item(i);
			NodeList child = n.getChildNodes();
			String str = new String();
			for (int j = 0; j < child.getLength(); j++) {
				NamedNodeMap rmap = ((Node)child.item(j)).getAttributes();
				str = str + rmap.getNamedItem("total_activity").getNodeValue() + "\t";
			}
			System.out.println(str);
		}
	}
	
	public void OutputTasks() {
		NodeList nl = data.GetNodeList(TIME);
		if (nl == null) {
			System.out.println("no time entires");
			return;
		}
		
		for (int i = 0; i < nl.getLength(); i++) {
			Node n = nl.item(i);
			NodeList child = n.getChildNodes();
			String str = new String();
			for (int j = 0; j < child.getLength(); j++) {
				NamedNodeMap rmap = ((Node)child.item(j)).getAttributes();
				str = str + rmap.getNamedItem("total_tasks").getNodeValue() + "\t";
			}
			System.out.println(str);
		}
	}
	
	public void OutputTimePeriod() {
		NodeList nl = data.GetNodeList(TIME);
		System.out.println(nl.getLength());
	}
	
	
	public void OutputMax(int max) {
		NodeList nl = data.GetNodeList(TIME);
		int nMax = 0;
		int i;
		int time = nl.getLength();
		double cost = 0.0;

		for (i = 0; i < nl.getLength(); i++) {
			Node n = nl.item(i);
			NodeList child = n.getChildNodes();
			NamedNodeMap rmap = ((Node)child.item(0)).getAttributes();
			int val = Integer.parseInt(rmap.getNamedItem("total_tasks").getNodeValue());
			if (val > max) {
				nMax++;
				if (nMax >= 5) time = Math.min(time, i);
			}
			cost += Double.parseDouble(n.getAttributes().getNamedItem("amtPaid").getNodeValue());
		}
		System.out.println(nl.getLength() + "\t" + cost + "\t" + time);
	}
	
	public void OutputOverhead() {
		NodeList nl = data.GetNodeList(TIME);
		int sumUnique = 0;
		int sumAssigned = 0;
		
		if (nl == null) {
			System.out.println("no time entires");
			return;
		}
		for (int i = 0; i < nl.getLength(); i++) {
			Node n = nl.item(i);
			NamedNodeMap map = n.getAttributes();
			sumUnique += Integer.parseInt(map.getNamedItem("numUniqueTasks").getNodeValue());
			sumAssigned += Integer.parseInt(map.getNamedItem("numAssignments").getNodeValue());
		}
		System.out.println((double)(sumAssigned - sumUnique)/sumUnique + "\t" + sumUnique + "\t" + sumAssigned);
	}
	
	public void OutputCost() {
		double total = 0.0;
		double a;
		int numPaid = 0;

		NodeList nl = data.GetNodeList(TIME);
		if (nl == null) {
			System.out.println("no time entires");
			return;
		}
		for (int i = 0; i < nl.getLength(); i++) {
			Node n = nl.item(i);
			NamedNodeMap map = n.getAttributes();
			a = Double.parseDouble(map.getNamedItem("amtPaid").getNodeValue());
			numPaid += Integer.parseInt(map.getNamedItem("numPaid").getNodeValue());
			total += a;
		}

		//System.out.println(total + "    " + numPaid);
		System.out.println(total/numPaid);
	}
	
	public static void main(String[] args) {
		if (args.length == 0) {
			System.out.println("Usage:  java Stats [o | a | t] statsfile");
		} else {
			Stats stats = new Stats(args[1]);
			if (args[0].equals("o"))
				stats.OutputOverhead();
			else if (args[0].equals("a"))
				stats.OutputActivity();
			else if (args[0].equals("t"))
				stats.OutputTimePeriod();
			else if (args[0].equals("m"))
				stats.OutputCost();
			else if (args[0].equals("f"))
				stats.OutputTasks();
			else if (args[0].charAt(0) == 'h') {
				stats.OutputMax(Integer.parseInt(args[0].substring(1)));
			}
		}
	}
}
