// XMLHandler.java

/**
 * @author Rishi Kacker and Matt Pauker
 * @version 1.0 05/13/2001
 */ 

import java.util.*;
import java.io.*;

import javax.xml.parsers.*;
import org.xml.sax.helpers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

public class XMLHandler
{
	private final static String DIRECTORY = "data";
	
	public final static int LOAD = 1;
	public final static int NEW = 2;
	
	private Document doc;
	private Element root;
	
	public XMLHandler(int type, String value) {
		
		switch (type) {
		case LOAD:
			doc = LoadXML(value);
			root = doc.getDocumentElement();
			break;
		case NEW:
			// Step 1: create a DocumentBuilderFactory and configure it
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			dbf.setValidating(false);

			// Step 2: create a DocumentBuilder that satisfies the constraints
			// specified by the DocumentBuilderFactory
			DocumentBuilder db = null;
			try {
				db = dbf.newDocumentBuilder();
			} catch (ParserConfigurationException pce) {
				System.err.println(pce);
				System.exit(1);
			}
		
			doc = db.newDocument();
		
			// Create the root node and add to the document
			root = doc.createElement(value);
			doc.appendChild(root);
			break;
		}
	}
	
	public NodeList GetNodeList(String name) {
		return doc.getElementsByTagName(name);
	}
	
	public Element CreateElement(String name) {
		return doc.createElement(name);
	}
	
	public Document GetDocument() {return doc;}
	
	public void Append(Element elem) {
		root.appendChild(elem);
	}
	
	public void SaveXML(String filename) {
		SaveXML(this.doc,filename);
	}
	
	public void SaveXML(Document doc, String filename) {
		try {
			File xmlfile = new File(DIRECTORY, filename);
			
			// Use a Transformer for output
			TransformerFactory tFactory = TransformerFactory.newInstance();
			Transformer transformer = tFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			StreamResult result = new StreamResult(new FileOutputStream(xmlfile));
			transformer.transform(source, result);
			
		} catch (TransformerConfigurationException tce) {
			// Error generated by the parser
			System.out.println ("\n** Transformer Factory error");
			System.out.println("   " + tce.getMessage() );
			// Use the contained exception, if any
			Throwable x = tce;
			if (tce.getException() != null)
				x = tce.getException();
			x.printStackTrace();
		} catch (TransformerException te) {
			// Error generated by the parser
			System.out.println ("\n** Transformation error");
			System.out.println("   " + te.getMessage() );
			// Use the contained exception, if any
			Throwable x = te;
			if (te.getException() != null)
				x = te.getException();
			x.printStackTrace();  
		} catch (IOException ioe) {
			System.out.println(ioe);
		}
	}
	
	public Document LoadXML(String filename)
	{
		// Step 1: create a DocumentBuilderFactory and configure it
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setValidating(false);

		// Step 2: create a DocumentBuilder that satisfies the constraints
		// specified by the DocumentBuilderFactory
		DocumentBuilder db = null;
		try {
			db = dbf.newDocumentBuilder();
		} catch (ParserConfigurationException pce) {
			System.err.println(pce);
			System.exit(1);
		}

		// Set an ErrorHandler before parsing
        //OutputStreamWriter errorWriter = 
	//		new OutputStreamWriter(System.err, outputEncoding);
        //db.setErrorHandler(
        //   new MyErrorHandler(new PrintWriter(errorWriter, true)));
		
		// Step 3: parse the input file
		Document doc = null;
		try {
			doc = db.parse(new File(DIRECTORY,filename));
		} catch (SAXException se) {
			System.err.println(se.getMessage());
			System.exit(1);
		} catch (IOException ioe) {
			System.err.println(ioe);
			System.exit(1);
		}
		
		return doc;
	}
	
	// Error handler to report errors and warnings
    private static class MyErrorHandler implements ErrorHandler {
        /** Error handler output goes here */
        private PrintStream out;

        MyErrorHandler(PrintStream out) {
            this.out = out;
        }

        /**
         * Returns a string describing parse exception details
         */
        private String getParseExceptionInfo(SAXParseException spe) {
            String systemId = spe.getSystemId();
            if (systemId == null) {
                systemId = "null";
            }
            String info = "URI=" + systemId +
                " Line=" + spe.getLineNumber() +
                ": " + spe.getMessage();
            return info;
        }

        // The following methods are standard SAX ErrorHandler methods.
        // See SAX documentation for more info.

        public void warning(SAXParseException spe) throws SAXException {
            out.println("Warning: " + getParseExceptionInfo(spe));
        }
        
        public void error(SAXParseException spe) throws SAXException {
            String message = "Error: " + getParseExceptionInfo(spe);
            throw new SAXException(message);
        }

        public void fatalError(SAXParseException spe) throws SAXException {
            String message = "Fatal Error: " + getParseExceptionInfo(spe);
            throw new SAXException(message);
        }
    }
}
